@@ -193,6 +193,9 @@ this.path <- function (verbose = getOption("verbose"))
     ## functions to get or check for an object in the n-th frame
     existsn <- function(x) exists(x, envir = sys.frame(n), inherits = FALSE)
     getn <- function(x) get(x, envir = sys.frame(n), inherits = FALSE)
+
+
+    ## function to save a path in the n-th frame
     assign.__file__ <- function(value = normalizePath(path, mustWork = TRUE)) {
         assign("__file__", value, envir = sys.frame(n))
     }
@@ -202,51 +205,38 @@ this.path <- function (verbose = getOption("verbose"))
     # for an appropriate source call (a call to function base::source,
     # base::sys.source, debugSource, or testthat::source_file)
     #
-    # an appropriate source call is a source call in which
-    #     argument 'file' has been evaluated (forced)
-    # this means, for example, the following is an inappropriate source call:
-    #     source(this.path())
-    # the argument 'file' is stored as a promise
-    #     containing the expression "this.path()"
-    # when the value of 'file' is requested, it assigns the value
-    #     returned by evaluating "this.path()" to variable 'file'
-    # there are two functions on the calling stack at
-    #     this point being 'source' and 'this.path'
-    # clearly, you don't want to request the 'file' argument from that source
-    #     call because the value of 'file' is under evaluation right now!
-    # the trick is to ask if variable ('ofile' for base::source, 'exprs' for base::sys.source)
-    #     exists in that function's evaluation environment. this is because that
-    #     variable is created AFTER argument 'file' has been forced
-    # if that variable does exist, then argument 'file' has been forced and the
-    #     source call is deemed appropriate. For base::source, the filename we want
-    #     is the variable 'ofile' from that function's evaluation environment. For
-    #     base::sys.source, the filename we want is the variable 'file' from that
-    #     function's evaluation environment.
-    # if that variable does NOT exist, then argument 'file' hasn't been forced and
-    #     the source call is deemed inappropriate. The 'for' loop moves to the next
-    #     function up the calling stack (if available)
+    # An appropriate source call is a source call in which argument 'file'
+    # ('fileName' in the case of debugSource, 'path' in the case of
+    # testthat::source_file) has been evaluated (forced). For example, the
+    # following is an inappropriate source call:
     #
-    # unfortunately, there is no way to check the argument 'fileName' has been forced
-    #     for 'debugSource' since all the work is done internally in C. Instead,
-    #     we have to use a 'tryCatch' statement. When we ask for an object by name
-    #     using 'get', R is capable of realizing if a variable is asking for its
-    #     own definition (a recursive definition). The exact error is "promise already
-    #     under evaluation" which indicates that the promise evaluation is requesting
-    #     its own value. So we use the 'tryCatch' to get the argument 'fileName'
-    #     from the evaluation environment of 'debugSource', and if it does not raise
-    #     an error, then we are safe to return that value. If not, the condition
-    #     returns false and the 'for' loop moves to the next function up the calling
-    #     stack (if available)
+    # source(this.path())
+    #
+    # The argument 'file' is stored as a promise containing the expression
+    # this.path(). When the value of 'file' is requested, it assigns the value
+    # returned by evaluating this.path() to variable 'file'
+    #
+    # There are two functions on the calling stack at this point being source
+    # and this.path. Clearly, you don't want to request the 'file' argument from
+    # that source call because the value of 'file' is under evaluation right
+    # now! The trick is to ask if a variable exists in that function's
+    # evaluation environment that is only created AFTER 'file' has been forced.
+    # For base::source, we ask if variable 'ofile' exists. For base::sys.source
+    # and testthat::source_file, we ask if variable 'exprs' exists. For
+    # debugSource, we cannot use this trick. Refer to the documentation below.
+    #
+    # If that variable exists, then argument 'file' has been forced and the
+    # source call is deemed appropriate. If that variable does not exist, then
+    # argument 'file' has not been forced and the source call is deemed
+    # inappropriate. The loop continues to the next iteration (if available)
 
 
     ## as of this.path_0.2.0, compatibility with 'debugSource' from the
     ## 'RStudio' environment was added. 'debugSource' presents challenges that
     ## other source functions do not. For example, it is impossible to test if
     ## argument 'fileName' has been forced since all of the work is done
-    ## internally in C. This is why a 'tryCatch' was used instead of an
-    ## 'existsn'. There are some things simpler in 'debugSource' than
-    ## 'base::source'. For instance, argument 'fileName' must be a character
-    ## vector while argument 'file' can be a character vector or a connection.
+    ## internally in C. This is why we use a 'tryCatch' statement instead of an
+    ## 'existsn'
     dbs <- if (.Platform$GUI == "RStudio")
         get("debugSource", mode = "function", "tools:rstudio",
             inherits = FALSE)
@@ -268,31 +258,85 @@ this.path <- function (verbose = getOption("verbose"))
 
     for (n in seq.int(sys.nframe(), 1L)[-1L]) {
         if (identical(sys.function(n), base::source)) {
+
+
+            ## if the argument 'file' to 'base::source' has not been forced,
+            ## continue to the next iteration
             if (!existsn("ofile"))
                 next
+
+
+            ## if the path has yet to be saved
             if (!existsn("__file__")) {
+
+
+                ## retrieve the unmodified 'file' argument
                 path <- getn("ofile")
-                if (is.character(path)) {
-                  if (path == "") {
-                    assign.__file__(NULL)
-                    next
-                  }
-                  con <- file(path, "r")
-                  on.exit(close(con))
-                  path <- summary.connection(con)$description
-                  on.exit()
-                  close(con)
-                }
-                else path <- summary.connection(path)$description
-                if (path %in% c("clipboard", "stdin")) {
-                  assign.__file__(NULL)
-                  next
-                }
+
+
+                ## as of this.path_0.3.0, compatibility was added when using
+                ## 'base::source' with argument 'chdir' set to TRUE.
+                ## this changes the working directory to the directory of the
+                ## 'file' argument. since the 'file' argument was relative to
+                ## the working directory prior to being changed, we need to
+                ## change it to the previous working directory before we can
+                ## open a connection (if necessary) and normalize the path
                 if (existsn("owd")) {
                   cwd <- getwd()
                   on.exit(setwd(cwd))
                   setwd(getn("owd"))
                 }
+
+
+                ## there are two options for 'file'
+                ## * connection
+                ## * character string
+                ## start with character string
+                if (is.character(path)) {
+
+
+                  ## use of "" refers to the R-level 'standard input' stdin.
+                  ## this means 'base::source' did not open a file, so we assign
+                  ## __file__ the value of NULL and continue to the next
+                  ## iteration. We use __file__ as NULL to skip this source call
+                  ## the next time this.path leads here
+                  if (path == "") {
+                    assign.__file__(NULL)
+                    next
+                  }
+
+
+                  ## as of this.path_0.4.3, the path is determined slightly
+                  ## differently. this change is to account for two possible
+                  ## scenarios
+                  ## * source("file://absolute or relative path")
+                  ## * source("file:///absolute path")
+                  ## the description of this connection should remove the
+                  ## leading characters
+                  con <- file(path, "r")
+                  on.exit(close(con), add = TRUE)
+                  path <- summary.connection(con)$description
+                  if (existsn("owd"))
+                    on.exit(setwd(cwd))
+                  else on.exit()
+                  close(con)
+                }
+                else path <- summary.connection(path)$description
+
+
+                ## use of "clipboard" and "stdin" refer to the clipboard or to
+                ## the C-level 'standard input' of the process, respectively.
+                ## this means 'base::source' did not open a file, so we assign
+                ## __file__ the value of NULL and continue to the next
+                ## iteration. We use __file__ as NULL to skip this source call
+                ## the next time this.path leads here
+                if (path %in% c("clipboard", "stdin")) {
+                  assign.__file__(NULL)
+                  next
+                }
+
+
+                ## assign __file__ as the absolute path
                 assign.__file__()
             }
             else if (is.null(getn("__file__")))
@@ -301,24 +345,57 @@ this.path <- function (verbose = getOption("verbose"))
             return(getn("__file__"))
         }
         else if (identical(sys.function(n), base::sys.source)) {
+
+
+            ## as with 'base::source', we check that argument 'file' has been
+            ## forced, and continue to the next iteration if not
             if (!existsn("exprs"))
                 next
+
+
+            ## much the same as 'base::source' except simpler, we don't have to
+            ## account for argument 'file' being a connection or ""
             if (!existsn("__file__")) {
                 path <- getn("file")
-                if (path %in% c("clipboard", "stdin"))
-                  stop(errorCondition("invalid 'file' argument, must not be \"clipboard\" nor \"stdin\"",
-                    call = sys.call(n)))
                 if (existsn("owd")) {
                   cwd <- getwd()
                   on.exit(setwd(cwd))
                   setwd(getn("owd"))
                 }
+
+
+                ## unlike 'base::source', 'base::sys.source' is intended to
+                ## source a file (not a connection), so we have to throw an
+                ## error if the user attempts to source a file named "clipboard"
+                ## or "stdin" since both of these DO NOT refer to files
+                if (path %in% c("clipboard", "stdin"))
+                  stop(errorCondition("invalid 'file' argument, must not be \"clipboard\" nor \"stdin\"",
+                    call = sys.call(n)))
                 assign.__file__()
             }
             where("call to function sys.source")
             return(getn("__file__"))
         }
         else if (identical(sys.function(n), dbs)) {
+
+
+            ## unlike 'base::source' and 'base::sys.source', there is no way to
+            ## check that argument 'fileName' has been forced, since all of the
+            ## work is done internally in C. Instead, we have to use a
+            ## 'tryCatch' statement. If argument 'fileName' has been forced, the
+            ## statement will proceed without an issue. If it has not, it is
+            ## because argument 'fileName' depends on itself recursively with
+            ## message "promise already under evaluation: recursive default
+            ## argument reference or earlier problems?". If you'd like to see,
+            ## try this:
+            ##
+            ## test <- function() get("fileName", sys.frame(1), inherits = FALSE)
+            ## debugSource(test())
+            ##
+            ## and you should see the error in which I'm referring. So the trick
+            ## is to use the 'tryCatch' statement to request argument
+            ## 'fileName', return TRUE if the statement proceeded without error
+            ## and FALSE if the statement produced an error.
             cond <- tryCatch({
                 path <- getn("fileName")
                 TRUE
@@ -326,6 +403,9 @@ this.path <- function (verbose = getOption("verbose"))
             if (!cond)
                 next
             if (!existsn("__file__")) {
+
+
+                ## we have to use 'enc2utf8' because 'debugSource' does as well
                 path <- enc2utf8(path)
                 if (path == "") {
                   assign.__file__(NULL)
@@ -348,18 +428,31 @@ this.path <- function (verbose = getOption("verbose"))
             return(getn("__file__"))
         }
         else if (identical(sys.function(n), sf)) {
+
+
+            ## as with 'base::source' and 'base::sys.source', we check that
+            ## argument 'path' has been forced, and continue to the next
+            ## iteration if not
             if (!existsn("exprs"))
                 next
             if (!existsn("__file__")) {
                 path <- getn("path")
-                if (path == "stdin")
-                  stop(errorCondition("invalid 'file' argument, must not be \"stdin\"",
-                    call = sys.call(n)))
                 if (existsn("old_dir")) {
                   cwd <- getwd()
                   on.exit(setwd(cwd))
                   setwd(getn("old_dir"))
                 }
+
+
+                ## like 'base::sys.source', 'testthat::source_file' is intended
+                ## to source a file (not a connection), so we have to throw an
+                ## error if the user attempts to source a file named "stdin"
+                ## since this DOES NOT refer to a file. However, we do not need
+                ## to check for "clipboard" since 'testthat::source_file'
+                ## handles that case correctly
+                if (path == "stdin")
+                  stop(errorCondition("invalid 'path' argument, must not be \"stdin\"",
+                    call = sys.call(n)))
                 assign.__file__()
             }
             where("call to function source_file in package testthat")
